// Copyright (c) Shinichi Yamagiwa, 2007
// Cortex-M3GDBX^u _E[hTvvO

#include "STM32_AddressDefs.h"

#define __EFI_ACR_Val 0x00000012
#define __RCC_CR_VAL  0x01000081
#define __RCC_CFGR_VAL 0x00288402
#define __NVIC_VTOR_VAL 0x00003000
#define __SYSTICK_CTRL_VAL 0x00000002
#define __SYSTICK_PERIOD 0x0000000A
#define CFGR_HPRE_MASK 0x000000F0 // Mask for AHB prescaler
#define __HCLKPRESC  0x00000000
#define __SYSTICKCLK (__HCLK>>3)
#define __HSI 8000000UL
#define CFGR_PLLMULL_MASK 0x003C0000 // Mask for PLL multiplier
#define __PLLMULL (((__RCC_CFGR_VAL & CFGR_PLLMULL_MASK) >> 18) + 2)
#define __SYSCLK  ((__HSI >> 1) * __PLLMULL)  // SYSCLK = HSI/2 * pllmul
#define __HCLK        (__SYSCLK)
#define CFGR_PRE2_MASK 0x00003800 // Mask for APB2 prescaler 
#define __PCLK2       (__HCLK)


#define __USART1_BAUDRATE         115200 //  4
#define __USART1_DATABITS         0x00000000
#define __USART1_STOPBITS         0x00000000
#define __USART1_PARITY           0x00000000
#define __USART1_FLOWCTRL         0x00000000

#define __DIV(__PCLK, __BAUD)       ((__PCLK*25)/(4*__BAUD))
#define __DIVMANT(__PCLK, __BAUD)   (__DIV(__PCLK, __BAUD)/100)
#define __DIVFRAQ(__PCLK, __BAUD)   (((__DIV(__PCLK, __BAUD) - (__DIVMANT(__PCLK, __BAUD) * 100)) * 16 + 50) / 100)
#define __USART_BRR(__PCLK, __BAUD) ((__DIVMANT(__PCLK, __BAUD) << 4)|(__DIVFRAQ(__PCLK, __BAUD) & 0x0F))

unsigned  int ADC_ConvertedValue;
unsigned  char Clock1s;
unsigned long timetick;
unsigned char leds = 0x01;
unsigned long ticks;

void SysTick_Handler (void) {
  if (ticks++ >= 99) { // Set Clock1s to 1 every 1 second
    ticks   = 0;
    Clock1s = 1;
  }

  // Blink the LEDs depending on ADC_ConvertedValue
  if (timetick++ >= ((ADC_ConvertedValue & 0xFFFF)>>8)) {
    timetick   = 0;
    leds ^= 1;                                   // toggle leds
    GPIOC_ODR = (GPIOC_ODR & 0xFFFFFFBF) | (leds << 6);
  }
}

void global_exception_handler(){
  //GPIOC_ODR = (GPIOC_ODR & 0xFFFFFFBF) | (GPIOC_ODR ^ ~0xFFFFFFBF);

  // Check SysTick Control Register (TICKINT bit)
  //  if(*((volatile unsigned int *)0xE000E010) & 0x2)
    SysTick_Handler();
}

int main(){

  int i;

#ifdef ROM_TARGET
  // EFI Configuration
  FLASH_ACR = __EFI_ACR_Val; // set access control register 
  
  // Clock Configuration
  RCC_CFGR = __RCC_CFGR_VAL; // set clock configuration register
  RCC_CR   = __RCC_CR_VAL; // set clock control register
  
  while ((RCC_CR & 0x00000002) == 0){} // Wait for HSIRDY = 1 (HSI is ready)
  while ((RCC_CR & 0x02000000) == 0){} // Wait for PLLRDY = 1 (PLL is ready)
  
  /* Wait till SYSCLK is stabilized (depending on selected clock) */
  while ((RCC_CFGR & 0xC) != ((__RCC_CFGR_VAL<<2) & 0xC));

  // NVIC Configuration
  SCB_ExceptionTableOffset = (__NVIC_VTOR_VAL & 0x1FFFFF80); // set register
#endif // ROM_TARGET

#if 0
  // SysTick Configuration
#if ((__SYSTICK_PERIOD*(__SYSTICKCLK/1000)-1) > 0xFFFFFF) // reload value to large
   #error "Reload Value to large! Please use 'HCLK/8' as System Timer clock source or smaller period"
#else
  SysTick_LOAD  = __SYSTICK_PERIOD*(__SYSTICKCLK/1000)-1; // set reload register
  SysTick_CTRL  = __SYSTICK_CTRL_VAL; // set clock source and Interrupt enable

  SysTick_VAL   =  0; // clear  the counter

  SysTick_CTRL |= 0x1; // enable the counter
#endif
#endif

  // GPIO Configuration
  // GPIO Port C
  RCC_APB2ENR |= 0x00000010; // enable clock for GPIOC
  GPIOC_CRL = 0x03000000; // set Port configuration register low
  GPIOC_CRH = 0x00000000; // set Port configuration register high

  while(1){
    for(i=0;i<500000;i++){}
    leds ^= 1;                                   // toggle leds
    GPIOC_ODR = (GPIOC_ODR & 0xFFFFFFBF) | (leds << 6);
  }
  while(1){}


  // Main
  // initialise A/D converter
  RCC_AHBENR |= (1<<0); // enable periperal clock for DMA
  DMA1_CMAR  = (unsigned int)&ADC_ConvertedValue; // set channel1 memory address
  DMA1_CPAR  = (unsigned int)&(ADC1_DR); // set channel1 peripheral address
  DMA1_CNDTR = 1; // transmit 1 word
  DMA1_CCR   = 0x00002520; // configure DMA channel
  DMA1_CCR  |= (1 << 0); // DMA Channel 1 enable
  RCC_APB2ENR |= (1<<9); // enable periperal clock for ADC1
  ADC1_SQR1 &= ~0x00F00000; // only one conversion
  ADC1_SMPR2 &= 0x00000038; // clear bits 3..5 (channel1)
  ADC1_SMPR2 |= 0x00000028; // set sample time (55,5 cycles)
  ADC1_SQR3  &= 0x0000001F; // clear bits 0..4
  ADC1_SQR3  |= 0x00000001; // set rank
  ADC1_CR1   =  0x00000100; // use independant mode, SCAN mode
  ADC1_CR2   =  0x000E0103; // use data align right,continuous conversion
  // enable ADC, DMA mode, no external Trigger
  ADC1_CR2  |=  0x00500000; // start SW conversion

  int AD_value;
  int AD_scaled_ex = 0;
  int AD_scaled;


  i = 0;
  while (1) { // Loop forever
    AD_value  = ADC_ConvertedValue; // Read AD value
    AD_scaled = AD_value >> 6; // AD value scaled to 0 .. 78 (lines on LCD)
    if (AD_scaled != AD_scaled_ex) { // If new AD value different than old
      AD_scaled_ex = AD_scaled;
    }

    if (Clock1s) {
      //      asm ("BKPT");
      Clock1s = 0;
    }
  }
  return 1;
}
